require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_armdeployment',
  dscmeta_resource_friendly_name: 'ARMDeployment',
  dscmeta_resource_name: 'POSHOrigin_ARMDeployment',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'POSHOrigin',
  dscmeta_module_version: '1.8.1',
  docs: 'The DSC ARMDeployment resource type.
         Automatically generated from version 1.8.1',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_deploymenttype: {
      type: "Optional[Enum['Incremental', 'Complete']]",
      desc: 'ARM deployment type',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Enum['Present', 'Absent']",
      desc: 'Ensure that the ARM deployment matches',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_name: {
      type: 'String',
      desc: 'Name of the ARM deployment',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_resourcegroup: {
      type: 'String',
      desc: 'Resource group to deploy ARM template to',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_credential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to connect to Azure Resource Manager',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_variables: {
      type: 'Optional[String]',
      desc: 'JSON string of variables to use in deployment',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_resources: {
      type: 'String',
      desc: 'JSON string of resources to deploy',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_version: {
      type: 'String',
      desc: 'Version of the ARM template',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
